from sqlalchemy import (Boolean, Column, DateTime, Integer, String)
from . import Base
from .game_stats_base import GameStatsBase


class TeamGameStats(GameStatsBase, Base):
    __tablename__ = 'team_game_stats'

    firstdowns_no = Column(Integer)
    firstdowns_pass = Column(Integer)
    firstdowns_rush = Column(Integer)
    firstdowns_penalty = Column(Integer)
    penalties_no = Column(Integer)
    penalties_yds = Column(Integer)
    conversions_thirdconv = Column(Integer)
    conversions_thirdatt = Column(Integer)
    conversions_fourthconv = Column(Integer)
    conversions_fourthatt = Column(Integer)
    time_of_poss = Column(Integer)
    redzone_att = Column(Integer)
    redzone_scores = Column(Integer)
    redzone_points = Column(Integer)
    redzone_tdrush = Column(Integer)
    redzone_tdpass = Column(Integer)
    redzone_fgmade = Column(Integer)
    redzone_endfga = Column(Integer)
    redzone_enddowns = Column(Integer)
    redzone_endint = Column(Integer)
    redzone_endfumb = Column(Integer)
    redzone_endhalf = Column(Integer)
    redzone_endgame = Column(Integer)
    misc_yds = Column(Integer)
    misc_top = Column(String)
    misc_ona = Column(Integer)
    misc_onm = Column(Integer)
    misc_ptsto = Column(Integer)


from sqlalchemy import (ForeignKey, Boolean, Column, DateTime, Integer, String)
from . import CreatedModifiedBase, Base
from sqlalchemy.orm import relationship


class PersonTeamSeasons(CreatedModifiedBase, Base):
    __tablename__ = 'person_team_seasons'
    
    person_team_seasons_id = Column(Integer, primary_key=True, autoincrement=True)
    person_id = Column(Integer, ForeignKey("people.person_id"))
    team_id = Column(Integer, ForeignKey("teams.team_id"))
    first_name = Column(String)
    last_name = Column(String)
    primary_position = Column(String)
    secondary_position = Column(String)
    role = Column(String, default='PLY')
    red_shirt = Column(Boolean, default=False)
    school_class = Column(String)
    team_sequence = Column(Boolean, primary_key=True)
    season = Column(Integer, primary_key=True)
    season_type = Column(String, primary_key=True)
    jersey_number = Column(Integer)
    short_name = Column(String)
    name = Column(String)
    check_name = Column(String)
    height = Column(Integer)
    weight = Column(Integer)
    game_class = Column(String, primary_key=True)

    team = relationship("Teams", foreign_keys=[team_id])
    person = relationship("Person", back_populates="seasons")
    names = relationship("PersonTeamSeasonsNames", back_populates="person_team_season")



from sqlalchemy import (Boolean, Column, DateTime, Integer, String)
from . import CreatedModifiedBase, Base
from sqlalchemy.orm import relationship


class Teams(CreatedModifiedBase, Base):
    __tablename__ = 'teams'

    team_id = Column(Integer, primary_key=True, autoincrement=True)
    r_id = Column(Integer)
    common_name = Column(String)
    known_names = Column(String)

    team_names = relationship("TeamNames", back_populates="team")



from sqlalchemy import (ForeignKey, Column, DateTime, Integer, String)
from . import CreatedModifiedBase, Base
from sqlalchemy.orm import relationship


class Venues(CreatedModifiedBase, Base):
    __tablename__ = 'venues'

    venue_id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String)
    known_names = Column(String)

    venue_names = relationship("VenueNames", back_populates="venue")

class VenueNames(CreatedModifiedBase, Base):
    __tablename__ = 'venue_names'

    venue_name_id = Column(Integer, primary_key=True, autoincrement=True)
    venue_id = Column(Integer, ForeignKey('venues.venue_id'))
    name = Column(String)

    venue = relationship("Venues", back_populates='venue_names')



from sqlalchemy import (Column, DateTime, Integer, Float, String)
from . import CreatedModifiedBase

class ConscGameStatsBase(CreatedModifiedBase, object):
    team_id = Column(Integer, primary_key=True)
    stat_type = Column(String, primary_key=True)
    stat_value = Column(Integer)
    start_game = Column(Integer, primary_key=True)
    end_game = Column(Integer, primary_key=True) # None means it is a current streak



from sqlalchemy import (Boolean, Column, DateTime, Integer, String, Time)
from . import CreatedModifiedBase, Base


class PxpEvent(CreatedModifiedBase, Base):
    __tablename__ = 'pxp_events'
    
    game_id = Column(Integer, primary_key=True)
    play_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, primary_key=True)
    event_type = Column(String, primary_key=True)
    yards = Column(Integer)
    is_score = Column(Boolean)
    event_subtype = Column(String)
    score_type = Column(String)
    secondary_person_id = Column(Integer)
    inside_20 = Column(Boolean)
    assisted = Column(Boolean)
    period = Column(Integer)
    direction = Column(String)
    in_air = Column(Integer)
    yards_after_catch = Column(Integer)
    


from sqlalchemy import (ForeignKey, Column, DateTime, Integer, String)
from . import CreatedModifiedBase, Base
from sqlalchemy.orm import relationship


class TeamNames(CreatedModifiedBase, Base):
    __tablename__ = 'team_names'

    team_name_id = Column(Integer, primary_key=True, autoincrement=True)
    team_id = Column(Integer, ForeignKey('teams.team_id'))
    name = Column(String)

    team = relationship("Teams", back_populates='team_names')



import sqlalchemy as db
import pandas as pd
from sqlalchemy.orm import sessionmaker

class Context:
    """Context class for connecting to SQL databases"""

    def __init__(self, server, username, password, database):

        self.server = server
        self.username = username
        self.password = password
        self.database = database

        engine = db.create_engine(f'mssql+pyodbc://{self.username}:{self.password}@{self.server}/{self.database}?driver=ODBC+Driver+17+for+SQL+Server')

        Session = sessionmaker(bind=engine)

        self.session = Session()

    def query(self, model):
        """Query a table"""
        return self.session.query(model)

    def query_join(self, model1, model2):
        """Query a table"""
        return self.session.query(model1, model2)

    def insert(self, record):
        self.session.add(record)

    def add_all(self, records):
        self.session.add_all(records)

    def merge(self, record):
        self.session.merge(record)

    def delete(self, record):
        self.session.delete(record)
        self.commit()

    def execute(self, statement, args=None):
        """Execute a raw SQL statement and return records"""
        if not args:
            return self.session.execute(statement)
        else:
            return self.session.execute(statement, args)


    def read_sql_query(self, statement):
        return pd.read_sql_query(statement, self.session.bind.engine)

    def save_changes(self):
        self.session.commit()

    def commit(self):
        self.session.commit()

    def rollback(self):
        self.session.rollback()

    def close(self):
        self.session.close()




from sqlalchemy import (ForeignKey, Column, DateTime, Integer, String)
from . import CreatedModifiedBase, Base
from sqlalchemy.orm import relationship


class PersonTeamSeasonsNames(CreatedModifiedBase, Base):
    __tablename__ = 'person_team_seasons_names'
    
    person_team_seasons_names_id = Column(Integer, primary_key=True)
    person_team_seasons_id = Column(Integer, ForeignKey("person_team_seasons.person_team_seasons_id"))
    checkname = Column(String)
    shortname = Column(String)

    person_team_season = relationship("PersonTeamSeasons", back_populates="names")



from sqlalchemy import (ForeignKey, Boolean, Column, DateTime, Integer, String)
from . import CreatedModifiedBase, Base
from sqlalchemy.orm import relationship


class TeamSeasons(CreatedModifiedBase, Base):
    __tablename__ = 'team_seasons'

    team_season_id = Column(Integer, primary_key=True, autoincrement=True)
    team_id = Column(Integer, ForeignKey('teams.team_id'))
    season = Column(Integer)
    season_type = Column(String)
    full_name = Column(String)
    conference_id = Column(Integer)
    r_id = Column(String)
    r_abbr = Column(String)
    r_name = Column(String)
    nickname = Column(String)
    division_id = Column(Integer)
    notes = Column(String)
    locale_name = Column(String)
    game_class = Column(String)
    primary_venue = Column(Integer)

    team = relationship("Teams", foreign_keys=[team_id])



from sqlalchemy import (Column, DateTime, Integer, Float)
from .consc_game_stats_base import ConscGameStatsBase

class PlayerConscGameStats(ConscGameStatsBase):
    __tablename__ = 'player_consc_game_stats'

    player_id = Column(Integer, primary_key=True)



from sqlalchemy import (Column, DateTime, Integer, Float, String)
from . import CreatedModifiedBase

class GameStatsBase(CreatedModifiedBase, object):
    # informational fields
    game_id = Column(Integer, primary_key=True)
    team_id = Column(Integer, primary_key=True)
    period = Column(Integer, primary_key=True)
    time_period = Column(String, primary_key=True)
    # rush stats
    rush_att = Column(Integer)
    rush_yds = Column(Integer)
    rush_td = Column(Integer)
    rush_long = Column(Integer)
    rush_gain = Column(Integer)
    rush_loss = Column(Integer)
    # rush averages
    rush_avg_yds_att = Column(Float)
    # pass stats
    pass_att = Column(Integer)
    pass_comp = Column(Integer)
    pass_td = Column(Integer)
    pass_int = Column(Integer)
    pass_yds = Column(Integer)
    pass_long = Column(Integer)
    pass_sacks = Column(Integer)
    pass_sackyds = Column(Integer)
    # pass averages
    pass_avg_yds_att = Column(Float)
    pass_avg_yds_comp = Column(Float)
    pass_pass_eff_rating = Column(Float)
    pass_td_int_ratio = Column(Float)
    pass_int_avoid_percent = Column(Float)
    pass_comp_percent = Column(Float)
    # kr stats
    kr_no = Column(Integer)
    kr_yds = Column(Integer)
    kr_td = Column(Integer)
    kr_long = Column(Integer)
    # kr averages
    kr_avg_yds_att = Column(Float)
    # pr stats
    pr_no = Column(Integer)
    pr_yds = Column(Integer)
    pr_td = Column(Integer)
    pr_long = Column(Integer)
    # pr averages
    pr_avg_yds_att = Column(Float)
    # ir stats
    ir_no = Column(Integer)
    ir_yds = Column(Integer)
    ir_td = Column(Integer)
    ir_long = Column(Integer)
    # ir averages
    ir_avg_yds_att = Column(Float)
    # fg stats
    fg_att = Column(Integer)
    fg_made = Column(Integer)
    fg_long = Column(Integer)
    fg_blkd = Column(Integer)
    # patstats
    pat_kickatt = Column(Integer)
    pat_kickmade = Column(Integer)
    pat_passatt = Column(Integer)
    pat_passmade = Column(Integer)
    pat_rushatt = Column(Integer)
    pat_rushmade = Column(Integer)
    pat_rcvatt = Column(Integer)
    pat_rcvmade = Column(Integer)
    pat_retfatt = Column(Integer)
    pat_retfmade = Column(Integer)
    pat_retkatt = Column(Integer)
    pat_retkmade = Column(Integer)
    # punt stats
    punt_no = Column(Integer)
    punt_yds = Column(Integer)
    punt_long = Column(Integer)
    punt_plus50 = Column(Integer)
    punt_inside20 = Column(Integer)
    punt_tb = Column(Integer)
    punt_fc = Column(Integer)
    punt_blkd = Column(Integer)
    # punt averages
    punt_avg = Column(Float)
    # ko stats
    ko_no = Column(Integer)
    ko_yds = Column(Integer)
    ko_tb = Column(Integer)
    ko_ob = Column(Integer)
    ko_fcyds = Column(Integer)
    # ko averages
    ko_avg_yds_att = Column(Float)
    # defense stats
    defense_tackua = Column(Integer)
    defense_tacka = Column(Integer)
    defense_tflua = Column(Integer)
    defense_tfla = Column(Integer)
    defense_tflyds = Column(Integer)
    defense_sacks = Column(Integer)
    defense_sackyds = Column(Integer)
    defense_brup = Column(Integer)
    defense_qbh = Column(Integer)
    defense_ff = Column(Integer)
    defense_fr = Column(Integer)
    defense_fryds = Column(Integer)
    defense_int = Column(Integer)
    defense_intyds = Column(Integer)
    defense_tot_tack = Column(Integer)
    defense_tflyds = Column(Integer)
    defense_sackyds = Column(Integer)
    defense_sackua = Column(Integer)
    defense_sacka = Column(Integer)
    defense_blkd = Column(Integer)
    defense_saf = Column(Integer)
    # scoring stats
    scoring_td = Column(Integer)
    scoring_patkick = Column(Integer)
    scoring_fg = Column(Integer)
    scoring_patrush = Column(Integer)
    scoring_patrcv = Column(Integer)
    scoring_total_pts = Column(Integer)
    scoring_saf = Column(Integer)
    scoring_patretkick = Column(Integer)
    scoring_patretfumb = Column(Integer)
    # fumbles
    fumbles_no = Column(Integer)
    fumbles_lost = Column(Integer)
    # fr
    fr_no = Column(Integer)
    fr_yds = Column(Integer)
    fr_td = Column(Integer)
    fr_long = Column(Integer)
    # field goal returns
    fgr_no = Column(Integer)
    fgr_yds = Column(Integer)



from sqlalchemy import (Boolean, Column, DateTime, Integer, String)
from . import CreatedModifiedBase, Base


class Game(CreatedModifiedBase, Base):
    __tablename__ = 'games'

    game_id = Column(Integer, primary_key=True, autoincrement=False)
    tgt_id = Column(Integer)
    opp_id = Column(Integer)
    tgt_abbr = Column(String)
    opp_abbr = Column(String)
    home = Column(Integer)
    visitors = Column(Integer)
    home_score = Column(Integer)
    visitors_score = Column(Integer)
    tgt_score = Column(Integer)
    opp_score = Column(Integer)
    date = Column(DateTime)
    date_display = Column(String)
    attendance = Column(Integer)
    venue_id = Column(Integer)
    start_time = Column(DateTime)
    end_time = Column(DateTime)
    weather = Column(String)
    leaguegame = Column(Boolean)
    tgt_rank = Column(Integer)
    opp_rank = Column(Integer)
    game_class = Column(String)
    season_type = Column(String)
    season = Column(Integer)
    result = Column(String)
    disposition = Column(String)
    url = Column(String)
    byu_game_nid = Column(Integer)
    day_night = Column(String)
    surface = Column(String)
    periods = Column(Integer)

    def __str__(self):
        return f'{self.game_id} {self.game_class} {self.teams}'



from sqlalchemy import (Boolean, Column, DateTime, Integer, String, Time)
from . import CreatedModifiedBase, Base


class Pxp(CreatedModifiedBase, Base):
    __tablename__ = 'pxp'
    
    game_id = Column(Integer, primary_key=True)
    play_id = Column(Integer, primary_key=True)
    drive_id = Column(Integer, primary_key=True)
    drive_play_id = Column(Integer, primary_key=True)
    type = Column(String)
    text = Column(String)
    possession = Column(String)
    down = Column(Integer)
    period = Column(Integer)
    to_go = Column(Integer)
    ball_on = Column(Integer)
    ball_on_after = Column(Integer)
    down_after = Column(Integer)
    to_go_after = Column(Integer)
    possession_after = Column(String)
    yards = Column(Integer)
    is_score = Column(Boolean)
    time = Column(Time)
    score_type = Column(String)
    


from sqlalchemy import (Boolean, Column, DateTime, Integer, String, Float)
from . import Base
from .game_stats_base import GameStatsBase


class PlayerGameStats(GameStatsBase, Base):
    __tablename__ = 'player_game_stats'

    person_id = Column(Integer, primary_key=True)
    gp = Column(Integer)
    gs = Column(Integer)
    # rcv stats
    rcv_no = Column(Integer)
    rcv_yds = Column(Integer)
    rcv_td = Column(Integer)
    rcv_long = Column(Integer)
    rcv_tgt = Column(Integer)
    # rcv averages
    rcv_avg_yds_att = Column(Float)
    # fr averages
    fr_avg_yds_att = Column(Float)

    
    


from sqlalchemy import (Boolean, Column, DateTime, Integer, String, text)
from . import CreatedModifiedBase, Base


class Linescore(CreatedModifiedBase, Base):
    __tablename__ = 'linescores'

    game_id = Column(Integer, primary_key=True)
    tgt_id = Column(Integer, primary_key=True)
    opp_id = Column(Integer, primary_key=True)
    tgt_score_period_1 = Column(Integer)
    tgt_score_period_2 = Column(Integer)
    tgt_score_period_3 = Column(Integer)
    tgt_score_period_4 = Column(Integer)
    tgt_score_period_5 = Column(Integer)
    tgt_score_cuml_1 = Column(Integer)
    tgt_score_cuml_2 = Column(Integer)
    tgt_score_cuml_3 = Column(Integer)
    tgt_score_cuml_4 = Column(Integer)
    tgt_score_cuml_5 = Column(Integer)
    opp_score_period_1 = Column(Integer)
    opp_score_period_2 = Column(Integer)
    opp_score_period_3 = Column(Integer)
    opp_score_period_4 = Column(Integer)
    opp_score_period_5 = Column(Integer)
    opp_score_cuml_1 = Column(Integer)
    opp_score_cuml_2 = Column(Integer)
    opp_score_cuml_3 = Column(Integer)
    opp_score_cuml_4 = Column(Integer)
    opp_score_cuml_5 = Column(Integer)



from sqlalchemy import (Boolean, Column, DateTime, Integer, String, text)
from . import CreatedModifiedBase, Base


class GameDrive(CreatedModifiedBase, Base):
    __tablename__ = 'game_drives'

    game_id = Column(Integer, primary_key=True)
    drive_id = Column(Integer, primary_key=True)
    possession = Column(Integer)
    start_how = Column(String)
    start_period = Column(Integer)
    start_time = Column(String)
    start_ball_on = Column(Integer)
    end_how = Column(String)
    end_period = Column(Integer)
    end_time = Column(String)
    end_ball_on = Column(Integer)
    plays = Column(Integer)
    yards = Column(Integer)
    time_of_possession = Column(String)
    redzone = Column(Integer)



from sqlalchemy import (Boolean, Column, DateTime, Integer, String)
from . import CreatedModifiedBase, Base
from sqlalchemy.orm import relationship


class Person(CreatedModifiedBase, Base):
    __tablename__ = 'people'

    person_id = Column(Integer, primary_key=True, autoincrement=True)
    first_name = Column(String)
    last_name = Column(String)
    middle_name = Column(String)
    birthdate = Column(DateTime)
    hometown_country = Column(String)
    hometown_city = Column(String)
    hometown_state = Column(String)
    high_school_id = Column(Integer)
    high_school_country = Column(String)
    high_school_city = Column(String)
    high_school_state = Column(String)
    high_school_name = Column(String)
    college_id = Column(Integer)
    college_country = Column(String)
    college_city = Column(String)
    college_state = Column(String)
    college_name = Column(String)
    r_id = Column(Integer)

    seasons = relationship("PersonTeamSeasons", back_populates="person")



from datetime import datetime
from sqlalchemy import (Boolean, Column, DateTime, Integer, String)

from sqlalchemy.orm import registry

mapper_registry = registry()
Base = mapper_registry.generate_base()

class CreatedModifiedBase(object):
    created = Column(DateTime, default=datetime.now())
    modified = Column(DateTime, default=datetime.now())



from sqlalchemy import (ForeignKey, Column, DateTime, Integer, String)
from . import CreatedModifiedBase, Base
from sqlalchemy.orm import relationship


class VenueSeasons(CreatedModifiedBase, Base):
    __tablename__ = 'venue_seasons'

    venue_season_id = Column(Integer, primary_key=True, autoincrement=True)
    season = Column(Integer)
    venue_id = Column(Integer, ForeignKey('venues.venue_id'))
    capacity = Column(Integer)
    full_name = Column(String)
    city = Column(String)
    state = Column(String)
    country = Column(String)
    zipcode = Column(String)
    timezone = Column(String)
    season_type = Column(String)
    game_class = Column(String)

    venue = relationship("Venues", foreign_keys=[venue_id])
